/*-
 *  Copyright 2020 Beuth Hochschule für Technik Berlin, Hochschule für Technik Stuttgart
 *
 *  This file is part of CityDoctor2.
 *
 *  CityDoctor2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CityDoctor2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with CityDoctor2.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.hft.stuttgart.citydoctor2.datastructure;

import de.hft.stuttgart.citydoctor2.check.Check;
import de.hft.stuttgart.citydoctor2.check.CheckError;
import de.hft.stuttgart.citydoctor2.check.CheckId;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import de.hft.stuttgart.citydoctor2.utils.CityGmlUtils;
import de.hft.stuttgart.citydoctor2.utils.CopyHandler;
import de.hft.stuttgart.citydoctor2.utils.Copyable;
import org.citygml4j.core.model.core.AbstractCityObject;
import org.citygml4j.core.model.core.AbstractSpace;
import org.citygml4j.core.model.transportation.AbstractTransportationSpace;
import org.citygml4j.core.model.transportation.AuxiliaryTrafficArea;
import org.citygml4j.core.model.transportation.TrafficArea;
import org.citygml4j.core.util.geometry.GeometryFactory;
import org.xmlobjects.gml.model.geometry.aggregates.MultiSurface;
import org.xmlobjects.gml.model.geometry.aggregates.MultiSurfaceProperty;

import java.io.Serial;
import java.util.ArrayList;
import java.util.List;

/**
 * Representation of a transportation object.
 *
 * @author Matthias Betz
 */
public abstract class TransportationObject extends CityObject {

    @Serial
    private static final long serialVersionUID = -2698907271726700390L;

    public enum TransportationType {
        ROAD, TRACK, RAILWAY, TRAFFIC_AREA, AUXILLIARY_TRAFFIC_AREA, TRANSPORTATION_COMPLEX, SQUARE, AUXILLIARY_TRAFFIC_SPACE,
        TRAFFIC_SPACE, WATERWAY, SECTION, INTERSECTION
    }

    private AbstractCityObject ato;
    protected TransportationType type;

    public TransportationObject(TransportationType type) {
        this.type = type;
    }

    protected TransportationObject() {
    }

    void setType(TransportationType type) {
        this.type = type;
    }

    @Override
    public FeatureType getFeatureType() {
        return FeatureType.TRANSPORTATION;
    }

    @Override
    public AbstractCityObject getGmlObject() {
        return ato;
    }


    public void setGmlObject(AbstractCityObject tc) {
        ato = tc;
    }


    public TransportationType getTransportationType() {
        return type;
    }

    @Override
    public String toString() {
        return "TransportationObject [id=" + getGmlId() + "]";
    }


    @Override
    public void fillValues(Copyable original, CopyHandler handler) {
        super.fillValues(original, handler);
        TransportationObject originalTo = (TransportationObject) original;
        ato = originalTo.ato;
    }

    @Override
    public Copyable createCopyInstance() {
        return null;
    }

}
